@@ -8,8 +8,14 @@
 /*
 /*	int	alldig(string)
 /*	const char *string;
+/*
+/*	int	allalnum(string)
+/*	const char *string;
 /* DESCRIPTION
 /*	alldig() determines if its argument is an all-numerical string.
+/*
+/*	allalnum() determines if its argument is an all-alphanumerical
+/*	string.
 /* SEE ALSO
 /*	An alldig() routine appears in Brian W. Kernighan, P.J. Plauger:
 /*	"Software Tools", Addison-Wesley 1976.
@@ -22,6 +28,11 @@
 /*	IBM T.J. Watson Research
 /*	P.O. Box 704
 /*	Yorktown Heights, NY 10598, USA
+/*
+/*	Wietse Venema
+/*	Google, Inc.
+/*	111 8th Avenue
+/*	New York, NY 10011, USA
 /*--*/
 
 /* System library. */
@@ -46,3 +57,17 @@ int     alldig(const char *string)
 	    return (0);
     return (1);
 }
+
+/* allalnum - return true if string is all alphanum */
+
+int allalnum(const char *string)
+{
+    const char *cp;
+
+    if (*string == 0)
+        return (0);
+    for (cp = string; *cp != 0; cp++)
+        if (!ISALNUM(*cp))
+            return (0);
+    return (1);
+}
